@@ -8,7 +8,8 @@ from furl import furl
 import cachetools.func
 from cachetools import cached, TTLCache
 import requests
-
+from urllib3.util.retry import Retry
+from requests.adapters import HTTPAdapter
 from .ratelimit import RateLimitError, rate_limit
 
 AUTH_URI = os.environ.get('AUTH_URI', 'localhost:5000/auth')
@@ -27,6 +28,16 @@ SKIP_CACHE_WINDOW_SEC = int(os.environ.get(
 
 AUTH_DISABLED = os.environ.get('AUTH_DISABLED', "false") == "true"
 
+
+retries = Retry(total=5,
+                backoff_factor=0.1,
+                status_forcelist=[ 500, 502, 503, 504 ])
+
+session = requests.Session()
+session.mount('https://' + AUTH_URI, HTTPAdapter(max_retries=retries))
+session.mount('https://' + INFO_URL, HTTPAdapter(max_retries=retries))
+
+
 r = None
 if USE_REDIS:
     import redis
@@ -46,7 +57,7 @@ def get_usernames(user_ids, token=None):
     if token is None:
         raise ValueError('missing token')
     if len(user_ids):
-        users_request = requests.get(f"https://{AUTH_URL}/api/v1/username?id={','.join(map(str, user_ids))}",
+        users_request = session.get(f"https://{AUTH_URL}/api/v1/username?id={','.join(map(str, user_ids))}",
                                      headers={
                                          'authorization': 'Bearer ' + token},
                                      timeout=5)
@@ -61,13 +72,11 @@ def get_usernames(user_ids, token=None):
     else:
         return []
 
-
 user_cache = TTLCache(maxsize=CACHE_MAXSIZE, ttl=CACHE_TTL)
 
-
 @cached(cache=user_cache)
 def user_cache_http(token):
-    user_request = requests.get(
+    user_request = session.get(
         f"https://{AUTH_URL}/api/v1/user/cache", headers={'authorization': 'Bearer ' + token})
     if user_request.status_code == 200:
         return user_request.json()
@@ -97,7 +106,7 @@ def is_root_public(table_id, root_id, token):
 
     url = f"https://{AUTH_URL}/api/v1/table/{table_id}/root/{root_id}/is_public"
 
-    req = requests.get(
+    req = session.get(
         url, headers={'authorization': 'Bearer ' + token}, timeout=5)
 
     if req.status_code == 200:
@@ -113,7 +122,7 @@ def table_has_public(table_id, token):
 
     url = f"https://{AUTH_URL}/api/v1/table/{table_id}/has_public"
 
-    req = requests.get(
+    req = session.get(
         url, headers={'authorization': 'Bearer ' + token}, timeout=5)
     if req.status_code == 200:
         return req.json()
@@ -124,7 +133,7 @@ def table_has_public(table_id, token):
 @cachetools.func.ttl_cache(maxsize=CACHE_MAXSIZE, ttl=CACHE_TTL)
 def dataset_from_table_id(service_namespace, table_id, token):
     url = f"https://{INFO_URL}/api/v2/tablemapping/service/{service_namespace}/table/{table_id}/permission_group"
-    req = requests.get(
+    req = session.get(
         url, headers={'authorization': 'Bearer ' + token}, timeout=5)
     if req.status_code == 200:
         return req.json()
@@ -212,7 +221,7 @@ def auth_required(func=None, *, required_permission=None, public_table_key=None,
 
             if auth_header:
                 if not auth_header.startswith('Bearer '):
-                    resp = flask.Response("Invalid Request", 400)
+                    resp = make_api_error(400, "bad_auth_header", "Authorization header must begin with \'Bearer\'")
                     resp.headers['WWW-Authenticate'] = 'Bearer realm="' + AUTHORIZE_URI + \
                         '", error="invalid_request", error_description="Header must begin with \'Bearer\'"'
                     return resp
@@ -221,7 +230,7 @@ def auth_required(func=None, *, required_permission=None, public_table_key=None,
 
             if programmatic_access:
                 if not token and not flask.g.public_access():
-                    resp = flask.Response("Unauthorized", 401)
+                    resp = make_api_error(401, "no_token", "Unauthorized - No Token Provided")
                     resp.headers['WWW-Authenticate'] = 'Bearer realm="' + \
                         AUTHORIZE_URI + '"'
                     return resp
@@ -243,7 +252,7 @@ def auth_required(func=None, *, required_permission=None, public_table_key=None,
             elif not programmatic_access and not flask.g.public_access():
                 return flask.redirect(AUTHORIZE_URI + '?redirect=' + quote(flask.request.url), code=302)
             elif not flask.g.public_access():
-                resp = flask.Response("Invalid/Expired Token", 401)
+                resp = make_api_error(401, "invalid_token", "Unauthorized - Token is Invalid or Expired")
                 resp.headers['WWW-Authenticate'] = 'Bearer realm="' + AUTHORIZE_URI + \
                     '", error="invalid_token", error_description="Invalid/Expired Token"'
                 return resp
